=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-03-07 02:18:45 +0000
+++ bzrlib/branch.py	2006-03-07 12:39:38 +0000
@@ -47,7 +47,7 @@
                            NoWorkingTree)
 import bzrlib.inventory as inventory
 from bzrlib.inventory import Inventory
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.osutils import (isdir, quotefn,
                             rename, splitpath, sha_file,
                             file_kind, abspath, normpath, pathjoin,
@@ -619,7 +619,9 @@
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = LockableFiles(branch_transport, 'branch-lock')
+        control_files = LockableFiles(branch_transport, 'branch-lock',
+                                      TransportLock)
+        control_files.create_lock()
         control_files.lock_write()
         try:
             for file, content in utf8_files:
@@ -669,9 +671,8 @@
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        lock_file = 'lock'
-        branch_transport.put(lock_file, StringIO()) # TODO get the file mode from the bzrdir lock files., mode=file_mode)
-        control_files = LockableFiles(branch_transport, 'lock')
+        control_files = LockableFiles(branch_transport, 'lock', TransportLock)
+        control_files.create_lock()
         control_files.lock_write()
         control_files.put_utf8('format', self.get_format_string())
         try:
@@ -695,7 +696,7 @@
             format = BranchFormat.find_format(a_bzrdir)
             assert format.__class__ == self.__class__
         transport = a_bzrdir.get_branch_transport(None)
-        control_files = LockableFiles(transport, 'lock')
+        control_files = LockableFiles(transport, 'lock', TransportLock)
         return BzrBranch5(_format=self,
                           _control_files=control_files,
                           a_bzrdir=a_bzrdir,

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-03-05 11:48:54 +0000
+++ bzrlib/builtins.py	2006-03-06 07:11:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -2213,12 +2213,12 @@
     
     In the future, uncommit will create a changeset, which can then
     be re-applied.
+    """
 
-    TODO: jam 20060108 Add an option to allow uncommit to remove unreferenced
-              information in 'branch-as-repostory' branches.
-    TODO: jam 20060108 Add the ability for uncommit to remove unreferenced
-              information in shared branches as well.
-    """
+    # TODO: jam 20060108 Add an option to allow uncommit to remove
+    # unreferenced information in 'branch-as-repostory' branches.
+    # TODO: jam 20060108 Add the ability for uncommit to remove unreferenced
+    # information in shared branches as well.
     takes_options = ['verbose', 'revision',
                     Option('dry-run', help='Don\'t actually make changes'),
                     Option('force', help='Say yes to all questions.')]
@@ -2271,10 +2271,32 @@
                 revno=revno)
 
 
+class cmd_break_lock(Command):
+    """Break a dead lock on a repository, branch or working directory.
+
+    CAUTION: Locks should only be broken when you are sure that the process
+    holding the lock has been stopped.
+    
+    example:
+        bzr break-lock .
+    """
+    takes_args = ['location']
+    takes_options = [Option('show',
+                            help="just show information on the lock, " \
+                                 "don't break it"),
+                    ]
+    def run(self, location, show=False):
+        d = bzrdir.BzrDir.open(location)
+        repo = d.open_repository()
+        if not repo.is_locked():
+            raise errors.ObjectNotLocked(repo)
+
+
+# command-line interpretation helper for merge-related commands
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
           this_dir=None, backup_files=False, merge_type=Merge3Merger,
-          file_list=None, show_base=False, reprocess=False, 
+          file_list=None, show_base=False, reprocess=False,
           pb=DummyProgress()):
     """Merge changes into a tree.
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-03-07 02:18:45 +0000
+++ bzrlib/bzrdir.py	2006-03-07 12:39:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 import bzrlib
 import bzrlib.errors as errors
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.osutils import safe_unicode
 from bzrlib.osutils import (
                             abspath,
@@ -536,7 +536,8 @@
         """See BzrDir.__init__."""
         super(BzrDirPreSplitOut, self).__init__(_transport, _format)
         self._control_files = LockableFiles(self.get_branch_transport(None),
-                                            'branch-lock')
+                                            'branch-lock',
+                                            TransportLock)
 
     def clone(self, url, revision_id=None, basis=None, force_new_repo=False):
         """See BzrDir.clone()."""
@@ -864,7 +865,7 @@
         # Since we don't have a .bzr directory, inherit the
         # mode from the root directory
         t = get_transport(url)
-        temp_control = LockableFiles(t, '')
+        temp_control = LockableFiles(t, '', TransportLock)
         temp_control._transport.mkdir('.bzr',
                                       # FIXME: RBC 20060121 dont peek under
                                       # the covers
@@ -880,8 +881,8 @@
                       ('branch-format', self.get_format_string()),
                       ]
         # NB: no need to escape relative paths that are url safe.
-        control.put(lock_file, StringIO(), mode=file_mode)
-        control_files = LockableFiles(control, lock_file)
+        control_files = LockableFiles(control, lock_file, TransportLock)
+        control_files.create_lock()
         control_files.lock_write()
         try:
             for file, content in utf8_files:

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-03-06 06:19:14 +0000
+++ bzrlib/commit.py	2006-03-07 12:39:38 +0000
@@ -230,8 +230,8 @@
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
-        self.revprops = {'branch-nick': self.branch.nick}
-        if revprops:
+        self.revprops = {}
+        if revprops is not None:
             self.revprops.update(revprops)
 
         self.work_tree.lock_write()

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-03-06 06:19:14 +0000
+++ bzrlib/errors.py	2006-03-07 12:39:38 +0000
@@ -300,11 +300,13 @@
         self.obj = obj
 
 
-class BranchNotLocked(LockError):
-    """Branch %(branch)r is not locked"""
-    def __init__(self, branch):
-        # XXX: sometimes called with a LockableFiles instance not a Branch
-        self.branch = branch
+class ObjectNotLocked(LockError):
+    """%(obj)r is not locked"""
+    # this can indicate that any particular object is not locked; see also
+    # LockNotHeld which means that a particular *lock* object is not held by
+    # the caller -- perhaps they should be unified.
+    def __init__(self, obj):
+        self.obj = obj
 
 
 class ReadOnlyObjectDirtiedError(ReadOnlyError):
@@ -346,12 +348,6 @@
         self.lock = lock
 
 
-class BranchNotLocked(LockError):
-    """Branch %(branch)r not locked"""
-    def __init__(self, branch):
-        self.branch = branch
-
-
 class PointlessCommit(BzrNewError):
     """No changes to commit"""
 

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2006-01-25 01:43:34 +0000
+++ bzrlib/lock.py	2006-03-03 07:31:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,19 +15,19 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-"""Locking wrappers.
+"""Locking using OS file locks or file existence.
 
-This only does local locking using OS locks for now.
+Note: This method of locking is generally deprecated in favour of LockDir, but
+is used to lock local WorkingTrees, and by some old formats.  It's accessed
+through Transport.lock_read(), etc.
 
 This module causes two methods, lock() and unlock() to be defined in
 any way that works on the current platform.
 
 It is not specified whether these locks are reentrant (i.e. can be
 taken repeatedly by a single process) or whether they exclude
-different threads in a single process.  
-
-Eventually we may need to use some kind of lock representation that
-will work on a dumb filesystem without actual locking primitives.
+different threads in a single process.  That reentrancy is provided by 
+LockableFiles.
 
 This defines two classes: ReadLock and WriteLock, which can be
 implemented in different ways on different platforms.  Both have an
@@ -250,10 +250,3 @@
         except ImportError:
             raise NotImplementedError("please write a locking method "
                                       "for platform %r" % sys.platform)
-
-
-
-
-
-
-

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-03-07 02:18:45 +0000
+++ bzrlib/lockable_files.py	2006-03-07 12:39:38 +0000
@@ -23,48 +23,78 @@
 from bzrlib.errors import LockError, ReadOnlyError
 from bzrlib.osutils import file_iterator, safe_unicode
 from bzrlib.symbol_versioning import *
-from bzrlib.symbol_versioning import deprecated_method, zero_eight
 from bzrlib.trace import mutter
 import bzrlib.transactions as transactions
 
+# XXX: The tracking here of lock counts and whether the lock is held is
+# somewhat redundant with what's done in LockDir; the main difference is that
+# LockableFiles permits reentrancy.
 
 class LockableFiles(object):
-    """Object representing a set of files locked within the same scope
-
-    _lock_mode
-        None, or 'r' or 'w'
-
-    _lock_count
-        If _lock_mode is true, a positive count of the number of times the
-        lock has been taken *by this process*.  Others may have compatible 
-        read locks.
-
-    _lock
-        Lock object from bzrlib.lock.
+    """Object representing a set of related files locked within the same scope.
+
+    These files are used by a WorkingTree, Repository or Branch, and should
+    generally only be touched by that object.
+
+    LockableFiles also provides some policy on top of Transport for encoding
+    control files as utf-8.
+
+    LockableFiles manage a lock count and can be locked repeatedly by
+    a single caller.  (The underlying lock implementation generally does not
+    support this.)
+
+    Instances of this class are often called control_files.
+    
+    This object builds on top of a Transport, which is used to actually write
+    the files to disk, and an OSLock or LockDir, which controls how access to
+    the files is controlled.  The particular type of locking used is set when
+    the object is constructed.  In older formats OSLocks are used everywhere.
+    in newer formats a LockDir is used for Repositories and Branches, and 
+    OSLocks for the local filesystem.
     """
 
-    _lock_mode = None
-    _lock_count = None
-    _lock = None
+    # _lock_mode: None, or 'r' or 'w'
+
+    # _lock_count: If _lock_mode is true, a positive count of the number of
+    # times the lock has been taken *by this process*.   
+    
     # If set to False (by a plugin, etc) BzrBranch will not set the
     # mode on created files or directories
     _set_file_mode = True
     _set_dir_mode = True
 
-    def __init__(self, transport, lock_name):
+    def __init__(self, transport, lock_name, lock_class):
+        """Create a LockableFiles group
+
+        :param transport: Transport pointing to the directory holding the 
+            control files and lock.
+        :param lock_name: Name of the lock guarding these files.
+        :param lock_class: Class of lock strategy to use: typically
+            either LockDir or TransportLock.
+        """
         object.__init__(self)
         self._transport = transport
         self.lock_name = lock_name
         self._transaction = None
         self._find_modes()
-
-    def __del__(self):
-        if self._lock_mode or self._lock:
-            # XXX: This should show something every time, and be suitable for
-            # headless operation and embedding
-            from warnings import warn
-            warn("file group %r was not explicitly unlocked" % self)
-            self._lock.unlock()
+        self._lock_mode = None
+        self._lock_count = 0
+        esc_name = self._escape(lock_name)
+        self._lock = lock_class(transport, esc_name, 
+                                file_modebits=self._file_mode,
+                                dir_modebits=self._dir_mode)
+
+    def create_lock(self):
+        """Create the lock.
+
+        This should normally be called only when the LockableFiles directory
+        is first created on disk.
+        """
+        self._lock.create()
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__,
+                           self._transport)
 
     def _escape(self, file_or_path):
         if not isinstance(file_or_path, basestring):
@@ -108,7 +138,7 @@
         """
 
         relpath = self._escape(file_or_path)
-        #TODO: codecs.open() buffers linewise, so it was overloaded with
+        # TODO: codecs.open() buffers linewise, so it was overloaded with
         # a much larger buffer, do we need to do the same for getreader/getwriter?
         if mode == 'rb': 
             return self.get(relpath)
@@ -170,8 +200,7 @@
                 raise ReadOnlyError(self)
             self._lock_count += 1
         else:
-            self._lock = self._transport.lock_write(
-                    self._escape(self.lock_name))
+            self._lock.lock_write()
             self._lock_mode = 'w'
             self._lock_count = 1
             self._set_transaction(transactions.WriteTransaction())
@@ -183,8 +212,7 @@
                    "invalid lock mode %r" % self._lock_mode
             self._lock_count += 1
         else:
-            self._lock = self._transport.lock_read(
-                    self._escape(self.lock_name))
+            self._lock.lock_read()
             self._lock_mode = 'r'
             self._lock_count = 1
             self._set_transaction(transactions.ReadOnlyTransaction())
@@ -194,15 +222,18 @@
     def unlock(self):
         # mutter("unlock: %s (%s)", self, self._lock_count)
         if not self._lock_mode:
-            raise errors.BranchNotLocked(self)
+            raise errors.LockNotHeld(self)
         if self._lock_count > 1:
             self._lock_count -= 1
         else:
             self._finish_transaction()
             self._lock.unlock()
-            self._lock = None
             self._lock_mode = self._lock_count = None
 
+    def is_locked(self):
+        """Return true if this LockableFiles group is locked"""
+        return self._lock_count >= 1
+
     def get_transaction(self):
         """Return the current active transaction.
 
@@ -229,3 +260,37 @@
         transaction = self._transaction
         self._transaction = None
         transaction.finish()
+
+
+class TransportLock(object):
+    """Locking method which uses transport-dependent locks.
+
+    On the local filesystem these transform into OS-managed locks.
+
+    These do not guard against concurrent access via different
+    transports.
+
+    This is suitable for use only in WorkingTrees (which are at present
+    always local).
+    """
+    def __init__(self, transport, escaped_name, file_modebits, dir_modebits):
+        self._transport = transport
+        self._escaped_name = escaped_name
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
+
+    def lock_write(self):
+        self._lock = self._transport.lock_write(self._escaped_name)
+
+    def lock_read(self):
+        self._lock = self._transport.lock_read(self._escaped_name)
+
+    def unlock(self):
+        self._lock.unlock()
+        self._lock = None
+
+    def create(self):
+        """Create lock mechanism"""
+        # for old-style locks, create the file now
+        self._transport.put(self._escaped_name, StringIO(), 
+                            mode=self._file_modebits)

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-02-22 07:06:12 +0000
+++ bzrlib/lockdir.py	2006-03-06 04:30:17 +0000
@@ -87,6 +87,7 @@
 >>> # typically will be obtained from a BzrDir, Branch, etc
 >>> t = MemoryTransport()
 >>> l = LockDir(t, 'sample-lock')
+>>> l.create()
 >>> l.wait_lock()
 >>> # do something here
 >>> l.unlock()
@@ -95,6 +96,7 @@
 
 import os
 import time
+from warnings import warn
 from StringIO import StringIO
 
 import bzrlib.config
@@ -128,6 +130,9 @@
 # TODO: Some kind of callback run while polling a lock to show progress
 # indicators.
 
+# TODO: Make sure to pass the right file and directory mode bits to all
+# files/dirs created.
+
 _DEFAULT_TIMEOUT_SECONDS = 300
 _DEFAULT_POLL_SECONDS = 0.5
 
@@ -136,7 +141,7 @@
 
     __INFO_NAME = '/info'
 
-    def __init__(self, transport, path):
+    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755):
         """Create a new LockDir object.
 
         The LockDir is initially unlocked - this just creates the object.
@@ -151,7 +156,11 @@
         self.transport = transport
         self.path = path
         self._lock_held = False
-        self._info_path = path + self.__INFO_NAME
+        self._fake_read_lock = False
+        self._held_dir = path + '/held'
+        self._held_info_path = self._held_dir + self.__INFO_NAME
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
         self.nonce = rand_chars(20)
 
     def __repr__(self):
@@ -161,26 +170,34 @@
 
     is_held = property(lambda self: self._lock_held)
 
+    def create(self):
+        """Create the on-disk lock.
+
+        This is typically only called when the object/directory containing the 
+        directory is first created.  The lock is not held when it's created.
+        """
+        if self.transport.is_readonly():
+            raise UnlockableTransport(self.transport)
+        self.transport.mkdir(self.path)
+
     def attempt_lock(self):
         """Take the lock; fail if it's already held.
         
         If you wish to block until the lock can be obtained, call wait_lock()
         instead.
         """
+        if self._fake_read_lock:
+            raise LockContention(self)
         if self.transport.is_readonly():
             raise UnlockableTransport(self.transport)
         try:
-            tmpname = '%s.pending.%s.tmp' % (self.path, rand_chars(20))
+            tmpname = '%s/pending.%s.tmp' % (self.path, rand_chars(20))
             self.transport.mkdir(tmpname)
             sio = StringIO()
             self._prepare_info(sio)
             sio.seek(0)
             self.transport.put(tmpname + self.__INFO_NAME, sio)
-            # FIXME: this turns into os.rename on posix, but into a fancy rename 
-            # on Windows that may overwrite existing directory trees.  
-            # NB: posix rename will overwrite empty directories, but not 
-            # non-empty directories.
-            self.transport.move(tmpname, self.path)
+            self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
             self.confirm()
             return
@@ -192,12 +209,15 @@
     def unlock(self):
         """Release a held lock
         """
+        if self._fake_read_lock:
+            self._fake_read_lock = False
+            return
         if not self._lock_held:
             raise LockNotHeld(self)
         # rename before deleting, because we can't atomically remove the whole
         # tree
-        tmpname = '%s.releasing.%s.tmp' % (self.path, rand_chars(20))
-        self.transport.rename(self.path, tmpname)
+        tmpname = '%s/releasing.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
         self._lock_held = False
         self.transport.delete(tmpname + self.__INFO_NAME)
         self.transport.rmdir(tmpname)
@@ -228,8 +248,8 @@
             return
         if current_info != dead_holder_info:
             raise LockBreakMismatch(self, current_info, dead_holder_info)
-        tmpname = '%s.broken.%s.tmp' % (self.path, rand_chars(20))
-        self.transport.rename(self.path, tmpname)
+        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
         # check that we actually broke the right lock, not someone else;
         # there's a small race window between checking it and doing the 
         # rename.
@@ -261,6 +281,10 @@
             raise LockBroken(self)
         
     def _read_info_file(self, path):
+        """Read one given info file.
+
+        peek() reads the info file of the lock holder, if any.
+        """
         return self._parse_info(self.transport.get(path))
 
     def peek(self):
@@ -271,7 +295,7 @@
         Otherwise returns None.
         """
         try:
-            info = self._read_info_file(self._info_path)
+            info = self._read_info_file(self._held_info_path)
             assert isinstance(info, dict), \
                     "bad parse result %r" % info
             return info
@@ -319,6 +343,25 @@
             else:
                 raise LockContention(self)
 
+    def lock_write(self):
+        """Wait for and acquire the lock."""
+        self.attempt_lock()
+
+    def lock_read(self):
+        """Compatability-mode shared lock.
+
+        LockDir doesn't support shared read-only locks, so this 
+        just pretends that the lock is taken but really does nothing.
+        """
+        # At the moment Branches are commonly locked for read, but 
+        # we can't rely on that remotely.  Once this is cleaned up,
+        # reenable this warning to prevent it coming back in 
+        # -- mbp 20060303
+        ## warn("LockDir.lock_read falls back to write lock")
+        if self._lock_held or self._fake_read_lock:
+            raise LockContention(self)
+        self._fake_read_lock = True
+
     def wait(self, timeout=20, poll=0.5):
         """Wait a certain period for a lock to be released."""
         # XXX: the transport interface doesn't let us guard 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-07 12:17:32 +0000
+++ bzrlib/repository.py	2006-03-07 12:39:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,14 +19,15 @@
 from unittest import TestSuite
 import xml.sax.saxutils
 
-
+import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
 from bzrlib.errors import InvalidRevisionId
 import bzrlib.gpg as gpg
 from bzrlib.inter import InterObject
 from bzrlib.knit import KnitVersionedFile
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
 from bzrlib.osutils import safe_unicode
 from bzrlib.revision import NULL_REVISION
 from bzrlib.store.versioned import VersionedFileStore, WeaveStore
@@ -162,6 +163,9 @@
     def lock_read(self):
         self.control_files.lock_read()
 
+    def is_locked(self):
+        return self.control_files.is_locked()
+
     @needs_read_lock
     def missing_revision_ids(self, other, revision_id=None):
         """Return the revision ids that other has that this does not.
@@ -811,13 +815,14 @@
 
         mutter('creating repository in %s.', a_bzrdir.transport.base)
         dirs = ['revision-store', 'weaves']
-        lock_file = 'branch-lock'
-        files = [('inventory.weave', StringIO(empty_weave)), 
+        files = [('inventory.weave', StringIO(empty_weave)),
                  ]
         
         # FIXME: RBC 20060125 dont peek under the covers
         # NB: no need to escape relative paths that are url safe.
-        control_files = LockableFiles(a_bzrdir.transport, 'branch-lock')
+        control_files = LockableFiles(a_bzrdir.transport, 'branch-lock',
+                                      TransportLock)
+        control_files.create_lock()
         control_files.lock_write()
         control_files._transport.mkdir_multi(dirs,
                 mode=control_files._dir_mode)
@@ -968,19 +973,18 @@
         """Create the required files and the initial control_files object."""
         # FIXME: RBC 20060125 dont peek under the covers
         # NB: no need to escape relative paths that are url safe.
-        lock_file = 'lock'
         repository_transport = a_bzrdir.get_repository_transport(self)
-        repository_transport.put(lock_file, StringIO()) # TODO get the file mode from the bzrdir lock files., mode=file_mode)
-        control_files = LockableFiles(repository_transport, 'lock')
+        control_files = LockableFiles(repository_transport, 'lock', LockDir)
+        control_files.create_lock()
         return control_files
 
     def _upload_blank_content(self, a_bzrdir, dirs, files, utf8_files, shared):
         """Upload the initial blank content."""
         control_files = self._create_control_files(a_bzrdir)
         control_files.lock_write()
-        control_files._transport.mkdir_multi(dirs,
-                mode=control_files._dir_mode)
         try:
+            control_files._transport.mkdir_multi(dirs,
+                    mode=control_files._dir_mode)
             for file, content in files:
                 control_files.put(file, content)
             for file, content in utf8_files:
@@ -1066,7 +1070,7 @@
             repo_transport = _override_transport
         else:
             repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock')
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
         text_store = self._get_text_store(repo_transport, control_files)
         control_store = self._get_control_store(repo_transport, control_files)
         _revision_store = self._get_revision_store(repo_transport, control_files)
@@ -1089,6 +1093,7 @@
      - a format marker of its own
      - an optional 'shared-storage' flag
      - an optional 'no-working-trees' flag
+     - a LockDir lock
     """
 
     def _get_control_store(self, repo_transport, control_files):
@@ -1145,7 +1150,7 @@
         
         self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)
         repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock')
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
         control_store = self._get_control_store(repo_transport, control_files)
         transaction = bzrlib.transactions.PassThroughTransaction()
         # trigger a write of the inventory store.
@@ -1169,7 +1174,7 @@
             repo_transport = _override_transport
         else:
             repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = LockableFiles(repo_transport, 'lock')
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
         text_store = self._get_text_store(repo_transport, control_files)
         control_store = self._get_control_store(repo_transport, control_files)
         _revision_store = self._get_revision_store(repo_transport, control_files)

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-03-04 13:03:57 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-03-06 07:11:09 +0000
@@ -37,6 +37,7 @@
                      'bzrlib.tests.blackbox.test_added',
                      'bzrlib.tests.blackbox.test_aliases',
                      'bzrlib.tests.blackbox.test_ancestry',
+                     'bzrlib.tests.blackbox.test_break_lock',
                      'bzrlib.tests.blackbox.test_bound_branches',
                      'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_checkout',

=== added file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2006-03-03 02:32:53 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for lock-breaking user interface"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.tests import TestCaseInTempDir
+
+class TestBreakLock(TestCaseInTempDir):
+    def test_break_lock_help(self):
+        self.run_bzr('break-lock', '--help')
+        # shouldn't fail
+
+    def test_show_no_lock(self):
+        wt = BzrDir.create_standalone_workingtree('.')
+        # out, err = self.run_bzr('break-lock', '--show', '.', retcode=3)
+        # shouldn't see any information
+        # self.assertContainsRe(err, 'not locked')

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-02-27 20:48:40 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-03-06 05:11:56 +0000
@@ -837,19 +837,17 @@
         self.runbzr('missing ../missing/new-branch')
 
     def test_external_command(self):
-        """test that external commands can be run by setting the path"""
+        """Test that external commands can be run by setting the path
+        """
+        # We don't at present run bzr in a subprocess for blackbox tests, and so 
+        # don't really capture stdout, only the internal python stream.
+        # Therefore we don't use a subcommand that produces any output or does
+        # anything -- we just check that it can be run successfully.  
         cmd_name = 'test-command'
-        output = 'Hello from test-command'
         if sys.platform == 'win32':
             cmd_name += '.bat'
-            output += '\r\n'
-        else:
-            output += '\n'
-
         oldpath = os.environ.get('BZRPATH', None)
-
         bzr = self.capture
-
         try:
             if os.environ.has_key('BZRPATH'):
                 del os.environ['BZRPATH']
@@ -859,7 +857,7 @@
                 f.write('@echo off\n')
             else:
                 f.write('#!/bin/sh\n')
-            f.write('echo Hello from test-command')
+            # f.write('echo Hello from test-command')
             f.close()
             os.chmod(cmd_name, 0755)
 
@@ -871,11 +869,6 @@
             os.environ['BZRPATH'] = '.'
 
             bzr(cmd_name)
-            # The test suite does not capture stdout for external commands
-            # this is because you have to have a real file object
-            # to pass to Popen(stdout=FOO), and StringIO is not one of those.
-            # (just replacing sys.stdout does not change a spawned objects stdout)
-            #self.assertEquals(bzr(cmd_name), output)
 
             # Make sure empty path elements are ignored
             os.environ['BZRPATH'] = os.pathsep

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-03-06 12:11:25 +0000
+++ bzrlib/tests/test_commit.py	2006-03-07 12:39:38 +0000
@@ -365,3 +365,15 @@
             self.assertEqual(['called', 'called'], calls)
         finally:
             del bzrlib.ahook
+
+    def test_commit_object_doesnt_set_nick(self):
+        # using the Commit object directly does not set the branch nick.
+        wt = self.make_branch_and_tree('.')
+        c = Commit()
+        c.commit(working_tree=wt, message='empty tree', allow_pointless=True)
+        self.assertEquals(wt.branch.revno(), 1)
+        self.assertEqual({},
+                         wt.branch.repository.get_revision(
+                            wt.branch.last_revision()).properties)
+
+

=== modified file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	2006-03-07 02:18:45 +0000
+++ bzrlib/tests/test_lockable_files.py	2006-03-07 12:39:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,8 @@
 
 from bzrlib.branch import Branch
 from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.lockdir import LockDir
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.transactions import (PassThroughTransaction,
                                  ReadOnlyTransaction,
@@ -26,14 +27,9 @@
                                  )
 from bzrlib.transport import get_transport
 
-class TestLockableFiles(TestCaseInTempDir):
 
-    def setUp(self):
-        super(TestLockableFiles, self).setUp()
-        transport = get_transport('.')
-        transport.mkdir('.bzr')
-        transport.put('.bzr/my-lock', StringIO(''))
-        self.lockable = LockableFiles(transport.clone('.bzr'), 'my-lock')
+# these tests are applied in each parameterized suite for LockableFiles
+class _TestLockableFiles_mixin(object):
 
     def test_read_write(self):
         self.assertRaises(NoSuchFile, self.lockable.get, 'foo')
@@ -94,3 +90,49 @@
     def test__escape_empty(self):
         self.assertEqual('', self.lockable._escape(''))
 
+
+# This method of adapting tests to parameters is different to 
+# the TestProviderAdapters used elsewhere, but seems simpler for this 
+# case.  
+class TestLockableFiles_TransportLock(TestCaseInTempDir,
+                                      _TestLockableFiles_mixin):
+
+    def setUp(self):
+        super(TestLockableFiles_TransportLock, self).setUp()
+        transport = get_transport('.')
+        transport.mkdir('.bzr')
+        sub_transport = transport.clone('.bzr')
+        self.lockable = LockableFiles(sub_transport, 'my-lock', TransportLock)
+        self.lockable.create_lock()
+        
+
+class TestLockableFiles_LockDir(TestCaseInTempDir,
+                              _TestLockableFiles_mixin):
+    """LockableFile tests run with LockDir underneath"""
+
+    def setUp(self):
+        super(TestLockableFiles_LockDir, self).setUp()
+        self.transport = get_transport('.')
+        self.lockable = LockableFiles(self.transport, 'my-lock', LockDir)
+        self.lockable.create_lock()
+
+    def test_lock_is_lockdir(self):
+        """Created instance should use a LockDir.
+        
+        This primarily tests the mixin adapter works properly.
+        """
+        ## self.assertIsInstance(self.lockable, LockableFiles)
+        ## self.assertIsInstance(self.lockable._lock_strategy,
+                              ## LockDirStrategy)
+
+    def test_lock_created(self):
+        self.assertTrue(self.transport.has('my-lock'))
+        self.lockable.lock_write()
+        self.assertTrue(self.transport.has('my-lock/held/info'))
+        self.lockable.unlock()
+        self.assertFalse(self.transport.has('my-lock/held/info'))
+        self.assertTrue(self.transport.has('my-lock'))
+
+
+    # TODO: Test the lockdir inherits the right file and directory permissions
+    # from the LockableFiles.

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2006-02-22 07:06:12 +0000
+++ bzrlib/tests/test_lockdir.py	2006-03-06 03:57:18 +0000
@@ -31,6 +31,12 @@
 # concurrent actors.  This is not a typical (or necessarily supported) use;
 # they're really meant for guarding between processes.
 
+# These tests are run on the default transport provided by the test framework
+# (typically a local disk transport).  That can be changed by the --transport
+# option to bzr selftest.  The required properties of the transport
+# implementation are tested separately.  (The main requirement is just that
+# they don't allow overwriting nonempty directories.)
+
 class TestLockDir(TestCaseWithTransport):
     """Test LockDir operations"""
 
@@ -58,6 +64,7 @@
         """Acquire and release a lock"""
         t = self.get_transport()
         lf = LockDir(t, 'test_lock')
+        lf.create()
         lf.attempt_lock()
         try:
             self.assertTrue(lf.is_held)
@@ -65,16 +72,24 @@
             lf.unlock()
             self.assertFalse(lf.is_held)
 
-    def test_11_lock_readonly_transport(self):
+    def test_11_create_readonly_transport(self):
+        """Fail to create lock on readonly transport"""
+        t = self.get_readonly_transport()
+        lf = LockDir(t, 'test_lock')
+        self.assertRaises(UnlockableTransport, lf.create)
+
+    def test_12_lock_readonly_transport(self):
         """Fail to lock on readonly transport"""
-        t = self.get_readonly_transport()
-        lf = LockDir(t, 'test_lock')
+        lf = LockDir(self.get_transport(), 'test_lock')
+        lf.create()
+        lf = LockDir(self.get_readonly_transport(), 'test_lock')
         self.assertRaises(UnlockableTransport, lf.attempt_lock)
 
     def test_20_lock_contested(self):
         """Contention to get a lock"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         lf2 = LockDir(t, 'test_lock')
         try:
@@ -92,6 +107,7 @@
         """Peek at the state of a lock"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         # lock is held, should get some info on it
         info1 = lf1.peek()
@@ -108,6 +124,7 @@
         """Peek over a readonly transport"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf2 = LockDir(self.get_readonly_transport(), 'test_lock')
         self.assertEqual(lf2.peek(), None)
         lf1.attempt_lock()
@@ -124,6 +141,7 @@
         """
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf2 = LockDir(t, 'test_lock')
         lf1.attempt_lock()
         try:
@@ -139,14 +157,15 @@
         """Succeed when waiting on a lock with no contention.
         """
         t = self.get_transport()
-        lf2 = LockDir(t, 'test_lock')
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         try:
             before = time.time()
-            lf2.wait_lock(timeout=0.4, poll=0.1)
+            lf1.wait_lock(timeout=0.4, poll=0.1)
             after = time.time()
             self.assertTrue(after - before <= 1.0)
         finally:
-            lf2.unlock()
+            lf1.unlock()
 
     def test_32_lock_wait_succeed(self):
         """Succeed when trying to acquire a lock that gets released
@@ -155,6 +174,7 @@
         """
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
 
         def wait_and_unlock():
@@ -181,6 +201,7 @@
         """
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
 
         def wait_and_unlock():
@@ -202,6 +223,7 @@
         """Confirm a lock that's already held"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         lf1.confirm()
 
@@ -209,12 +231,14 @@
         """Confirm a lock that's already held"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         self.assertRaises(LockNotHeld, lf1.confirm)
 
     def test_42_confirm_broken_manually(self):
         """Confirm a lock broken by hand"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         t.move('test_lock', 'lock_gone_now')
         self.assertRaises(LockBroken, lf1.confirm)
@@ -223,6 +247,7 @@
         """Break a lock whose caller has forgotten it"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         # we incorrectly discard the lock object without unlocking it
         del lf1
@@ -239,6 +264,7 @@
         """Lock break races with regular release"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         # someone else sees it's still locked
         lf2 = LockDir(t, 'test_lock')
@@ -255,6 +281,7 @@
         """Lock break races with someone else acquiring it"""
         t = self.get_transport()
         lf1 = LockDir(t, 'test_lock')
+        lf1.create()
         lf1.attempt_lock()
         # someone else sees it's still locked
         lf2 = LockDir(t, 'test_lock')
@@ -267,3 +294,26 @@
         self.assertRaises(LockBreakMismatch, lf2.force_break,
                           holder_info)
         lf3.unlock()
+
+    def test_46_fake_read_lock(self):
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        lf1.lock_read()
+        lf1.unlock()
+
+    def test_50_lockdir_representation(self):
+        """Check the on-disk representation of LockDirs is as expected.
+
+        There should always be a top-level directory named by the lock.
+        When the lock is held, there should be a lockname/held directory 
+        containing an info file.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+        lf1.create()
+        self.assertTrue(t.has('test_lock'))
+        lf1.lock_write()
+        self.assertTrue(t.has('test_lock/held/info'))
+        lf1.unlock()
+        self.assertFalse(t.has('test_lock/held/info'))

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2006-02-15 22:01:55 +0000
+++ bzrlib/tests/test_permissions.py	2006-03-06 05:01:35 +0000
@@ -37,7 +37,7 @@
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import get_transport
@@ -157,31 +157,31 @@
         try:
             transport = get_transport(self.get_url())
             transport.put('my-lock', StringIO(''))
-            lockable = LockableFiles(transport, 'my-lock')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertNotEqual(None, lockable._dir_mode)
             self.assertNotEqual(None, lockable._file_mode)
 
             LockableFiles._set_dir_mode = False
             transport = get_transport('.')
-            lockable = LockableFiles(transport, 'my-lock')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertEqual(None, lockable._dir_mode)
             self.assertNotEqual(None, lockable._file_mode)
 
             LockableFiles._set_file_mode = False
             transport = get_transport('.')
-            lockable = LockableFiles(transport, 'my-lock')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertEqual(None, lockable._dir_mode)
             self.assertEqual(None, lockable._file_mode)
 
             LockableFiles._set_dir_mode = True
             transport = get_transport('.')
-            lockable = LockableFiles(transport, 'my-lock')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertNotEqual(None, lockable._dir_mode)
             self.assertEqual(None, lockable._file_mode)
 
             LockableFiles._set_file_mode = True
             transport = get_transport('.')
-            lockable = LockableFiles(transport, 'my-lock')
+            lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertNotEqual(None, lockable._dir_mode)
             self.assertNotEqual(None, lockable._file_mode)
         finally:

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-03-07 12:17:32 +0000
+++ bzrlib/tests/test_repository.py	2006-03-07 12:39:38 +0000
@@ -157,7 +157,6 @@
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Repository format 7',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
         self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
         self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
         self.assertEqualDiff('# bzr weave file v5\n'
@@ -170,15 +169,14 @@
         repo = repository.RepositoryFormat7().initialize(control, shared=True)
         # we want:
         # format 'Bazaar-NG Repository format 7'
-        # lock ''
         # inventory.weave == empty_weave
         # empty revision-store directory
         # empty weaves directory
         # a 'shared-storage' marker file.
+        # lock is not present when unlocked
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Repository format 7',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
         self.assertEqualDiff('', t.get('shared-storage').read())
         self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
         self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
@@ -186,6 +184,34 @@
                              'w\n'
                              'W\n',
                              t.get('inventory.weave').read())
+        self.assertFalse(t.has('branch-lock'))
+
+    def test_creates_lockdir(self):
+        """Make sure it appears to be controlled by a LockDir existence"""
+        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        # TODO: Should check there is a 'lock' toplevel directory, 
+        # regardless of contents
+        self.assertFalse(t.has('lock/held/info'))
+        repo.lock_write()
+        self.assertTrue(t.has('lock/held/info'))
+
+    def test_uses_lockdir(self):
+        """repo format 7 actually locks on lockdir"""
+        base_url = self.get_url()
+        control = bzrdir.BzrDirMetaFormat1().initialize(base_url)
+        repo = repository.RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        repo.lock_write()
+        repo.unlock()
+        del repo
+        # make sure the same lock is created by opening it
+        repo = repository.Repository.open(base_url)
+        repo.lock_write()
+        self.assertTrue(t.has('lock/held/info'))
+        repo.unlock()
+        self.assertFalse(t.has('lock/held/info'))
 
     def test_shared_no_tree_disk_layout(self):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
@@ -201,7 +227,7 @@
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Repository format 7',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
+        ## self.assertEqualDiff('', t.get('lock').read())
         self.assertEqualDiff('', t.get('shared-storage').read())
         self.assertEqualDiff('', t.get('no-working-trees').read())
         repo.set_make_working_trees(True)
@@ -224,14 +250,15 @@
         repo.unlock()
         # we want:
         # format 'Bazaar-NG Knit Repository Format 1'
-        # lock ''
+        # lock: is a directory
         # inventory.weave == empty_weave
         # empty revision-store directory
         # empty weaves directory
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
         self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
         self.check_knits(t)
 
@@ -255,7 +282,7 @@
         repo = repository.RepositoryFormatKnit1().initialize(control, shared=True)
         # we want:
         # format 'Bazaar-NG Knit Repository Format 1'
-        # lock ''
+        # lock: is a directory
         # inventory.weave == empty_weave
         # empty revision-store directory
         # empty weaves directory
@@ -263,7 +290,8 @@
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
         self.assertEqualDiff('', t.get('shared-storage').read())
         self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
         self.check_knits(t)
@@ -282,7 +310,8 @@
         t = control.get_repository_transport(None)
         self.assertEqualDiff('Bazaar-NG Knit Repository Format 1',
                              t.get('format').read())
-        self.assertEqualDiff('', t.get('lock').read())
+        # XXX: no locks left when unlocked at the moment
+        # self.assertEqualDiff('', t.get('lock').read())
         self.assertEqualDiff('', t.get('shared-storage').read())
         self.assertEqualDiff('', t.get('no-working-trees').read())
         repo.set_make_working_trees(True)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-03-05 11:48:54 +0000
+++ bzrlib/workingtree.py	2006-03-06 22:11:12 +0000
@@ -62,7 +62,7 @@
                            NoSuchFile,
                            NotVersionedError)
 from bzrlib.inventory import InventoryEntry, Inventory
-from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.merge import merge_inner, transform_tree
 from bzrlib.osutils import (
                             abspath,
@@ -255,7 +255,7 @@
             assert isinstance(self._format, WorkingTreeFormat3)
             self._control_files = LockableFiles(
                 self.bzrdir.get_workingtree_transport(None),
-                'lock')
+                'lock', TransportLock)
 
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
@@ -454,13 +454,18 @@
             tree.set_last_revision(revision_id)
 
     @needs_write_lock
-    def commit(self, *args, **kwargs):
+    def commit(self, message=None, revprops=None, *args, **kwargs):
+        # avoid circular imports
         from bzrlib.commit import Commit
+        if revprops is None:
+            revprops = {}
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = self.branch.nick
         # args for wt.commit start at message from the Commit.commit method,
         # but with branch a kwarg now, passing in args as is results in the
         #message being used for the branch
-        args = (DEPRECATED_PARAMETER, ) + args
-        Commit().commit(working_tree=self, *args, **kwargs)
+        args = (DEPRECATED_PARAMETER, message, ) + args
+        Commit().commit(working_tree=self, revprops=revprops, *args, **kwargs)
         self._set_inventory(self.read_working_inventory())
 
     def id2abspath(self, file_id):
@@ -1429,7 +1434,7 @@
         if not isinstance(a_bzrdir.transport, LocalTransport):
             raise errors.NotLocalUrl(a_bzrdir.transport.base)
         transport = a_bzrdir.get_workingtree_transport(self)
-        control_files = LockableFiles(transport, 'lock')
+        control_files = LockableFiles(transport, 'lock', TransportLock)
         control_files.put_utf8('format', self.get_format_string())
         branch = a_bzrdir.open_branch()
         if revision_id is None:

